#ifndef _XmdReplicant_h
#define _XmdReplicant_h

#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <Xm/Xm.h>
#include <Xmd/String.h>

#define XmdREPLICANT_VERSION 0

/* XmdReplicantState contains state information for a replicant instance. */
typedef struct _XmdReplicantState XmdReplicantState;

typedef int    (*XmdReplicantFnVersion)   ();
typedef void   (*XmdReplicantFnConstruct) ();
typedef void   (*XmdReplicantFnDestruct)  ();
typedef Widget (*XmdReplicantFnCreate)    (
	XtAppContext application, Widget parent, XmdReplicantState *state);

/* XmdReplicantOpen opens a replicant from the file located at path. Replicant
   files hold state and point to shared object files containing their code.
   Replicant shared objects are searched for in $XMD_REPLICANT_PATH. Here is an
   example file that describes a calculator replicant:
   
     [Calculator]
     display=90
     history=1 + 2 = 3;9 * 10 = 90;
     mode=scientific
   
   When a replicant's code is needed, it is automatically loaded by libXmd and
   stays resident until all replicants using it have been freed. A system of
   reference counting is used to accomplish this. =
   
   Replicants must implement these symbols, where NAME is the base name of the
   replicant's shared object file (case sensitive, and excluding the .so
   extention):
   
   int NAME_XmdReplicantVersion ();
   Version must return the value of XmdREPLICANT_VERSION. This is used to detect
   the API version that the replicant was compiled with, and applications with a
   different API version will refuse to load it.

   void NAME_XmdReplicantConstruct ();
   Optional. Construct is called when the shared object is loaded.
   
   void NAME_XmdReplicantDestruct ();
   Optional. Descruct is called before the shared object is unloaded.
   
   Widget NAME_XmdReplicantCreate (
   	XtAppContext application, Widget parent, XmdReplicantState *state);
   Create instantiates a new widget representing the replicant given by state.
   */
Widget XmdReplicantOpen (XtAppContext app, Widget parent, ConstString path);

/* XmdReplicantResolveName returns the file path of the given replicant shared
   object name according to $XMD_REPLICANT_PATH. The returned string must be
   freed using XtFree(). */
String XmdReplicantResolveName (ConstString name);

/* XmdReplicantStateQuery queries the replicant's state file for a named value.
   The resulting string must be free'd using XtFree(). NULL is returned if there
   is no value associated with the given name. This may perform a read from
   disk. */
String XmdReplicantStateQuery (
	XmdReplicantState *state,
	ConstString name);

/* XmdReplicantStateStore stores a named value in the replicant's state file.
   This may perform a write to disk. */
void XmdReplicantStateStore (
	XmdReplicantState *state,
	ConstString name, ConstString value);

#endif
